"""
This package provides convenient utilities and data to write a sphinx config file.
"""

from __future__ import annotations

from pathlib import Path
import json
from typing import Dict, Tuple, Set, Optional, cast

# I'm thinking of going YEAR.month.releasenumber

__version__ = "0.0.4"

registry_file = Path(__file__).parent / "registry.json"


def get_intersphinx_mapping(
    *, only: Optional[Set[str]] = None
) -> Dict[str, Tuple[str, Optional[str]]]:
    """
    Return values of intersphinx_mapping for sphinx configuration.

    For convenience, the return dict is a copy so should be ok to mutate

    Parameters
    ----------
    only: Set of Str
        list of libraries to include.
        This is purely for optimisation as sphinx may download and load all the
        `objects.inv` listed, in get_intersphinx_mapping. This let users reduce
        the number of requested files.
    """
    mapping = cast(
        Dict[str, Tuple[str, Optional[str]]],
        {k: tuple(v) for (k, v) in json.loads(registry_file.read_bytes()).items()},
    )
    if only is None:
        return mapping
    else:
        missing = set(only) - set(mapping)
        if missing:
            raise ValueError(f"Missing libraries in 'only': {repr(sorted(missing))}")
        return {k: v for k, v in mapping.items() if k in only}



from intersphinx_registry import get_intersphinx_mapping
import pytest
import requests

MAPPING = get_intersphinx_mapping()
keys = set(MAPPING.keys())


@pytest.mark.parametrize("key", list(sorted(keys - {"jinja"})))
def test_format(key: str):
    assert isinstance(key, str)
    for v in MAPPING[key]:
        if v is None:
            continue
        assert v.startswith("https://"), v
        assert v.endswith("/"), v
        assert "readthedocs.org" not in v
        requests.head(v + "objects.inv").raise_for_status()


@pytest.mark.parametrize("key", list(sorted(keys)))
def test_reach_objects_inv(key: str):
    for v in MAPPING[key]:
        if v is None:
            continue
        requests.head(v + "objects.inv").raise_for_status()


def test_bad():
    with pytest.raises(ValueError, match="Missing libraries"):
        get_intersphinx_mapping(only={"-nonexistent-"})
